==> SSLeay-0.6.0.tar.gz: SSLeay-0.6.0/crypto/bn/bn_sqr.c <==
/* crypto/bn/bn_sqr.c */
/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
 * All rights reserved.
 * 
 * This file is part of an SSL implementation written
 * by Eric Young (eay@mincom.oz.au).
 * The implementation was written so as to conform with Netscapes SSL
 * specification.  This library and applications are
 * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the following conditions are aheared to.
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.  If this code is used in a product,
 * Eric Young should be given attribution as the author of the parts used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Eric Young (eay@mincom.oz.au)
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include "cryptlib.h"
#include "bn.h"
#include "bn_lcl.h"

/* r must not be a */
int BN_sqr(r, a, ctx)
BIGNUM *r;
BIGNUM *a;
BN_CTX *ctx;
	{
	int i,j,max,al;
	BIGNUM *tmp;
	BN_ULONG *ap,*rp;

	tmp=ctx->bn[ctx->tos];

	al=a->top;
	if (al == 0)
		{
		r->top=0;
		return(1);
		}

	max=(al*2+1);
	BN_clear(r);
	BN_clear(tmp);
	if (bn_expand(r,(max+1)*BN_BITS2) == NULL) return(0);
	if (bn_expand(tmp,(max+1)*BN_BITS2) == NULL) return(0);
	r->top=max;
	r->neg=a->neg;
	ap=a->d;
	rp=r->d;

	rp++;
	j=al;
	for (i=1; i<al; i++)
		{
		BN_ULONG r;

		j--;
		r= *(ap++);
		rp[j]+=bn_mul_add_word(rp,ap,j,r);
		rp+=2;
		}
	bn_fix_top(r);
	if (!BN_lshift1(r,r)) return(0);

	bn_sqr_words(tmp->d,a->d,a->top);
	tmp->top=al*2+1;
	bn_fix_top(tmp);
	if (!BN_add(r,r,tmp)) return(0);
	return(1);
	}

==> SSLeay-0.6.1.tar.gz: SSLeay-0.6.1/crypto/bn/bn_sqr.c <==
/* crypto/bn/bn_sqr.c */
/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
 * All rights reserved.
 * 
 * This file is part of an SSL implementation written
 * by Eric Young (eay@mincom.oz.au).
 * The implementation was written so as to conform with Netscapes SSL
 * specification.  This library and applications are
 * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the following conditions are aheared to.
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.  If this code is used in a product,
 * Eric Young should be given attribution as the author of the parts used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Eric Young (eay@mincom.oz.au)
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include "cryptlib.h"
#include "bn.h"
#include "bn_lcl.h"

/* r must not be a */
/* I've just gone over this and it is now %20 faster on x86 - eay - 27 Jun 96 */
int BN_sqr(r, a, ctx)
BIGNUM *r;
BIGNUM *a;
BN_CTX *ctx;
	{
	int i,j,max,al;
	BIGNUM *tmp;
	BN_ULONG *ap,*rp,c;

	tmp=ctx->bn[ctx->tos];

	al=a->top;
	if (al == 0)
		{
		r->top=0;
		return(1);
		}

	max=(al*2);
	if (bn_expand(r,max*BN_BITS2) == NULL) return(0);
	if (bn_expand(tmp,max*BN_BITS2) == NULL) return(0);

	r->neg=0;

	ap=a->d;
	rp=r->d;
	rp[0]=rp[max-1]=0;
	rp++;
	j=al;

	if (--j > 0)
		{
		ap++;
		rp[j]=bn_mul_word(rp,ap,j,ap[-1]);
		rp+=2;
		}

	for (i=2; i<al; i++)
		{
		j--;
		ap++;
		rp[j]=bn_mul_add_word(rp,ap,j,ap[-1]);
		rp+=2;
		}

	/* inlined shift */
	j=max;
	rp=r->d;
	c=0;
	for (i=0; i<j; i++)
		{
		BN_ULONG t;

		t= *rp;
		*(rp++)=((t<<1)|c)&BN_MASK2;
		c=(t & BN_TBIT)?1:0;
		}
	/* there will not be a carry */

	bn_sqr_words(tmp->d,a->d,al);

	/* inlined add */
	ap=tmp->d;
	rp=r->d;
	c=0;
	j=max;
	for (i=0; i<j; i++)
		{
		BN_ULONG t1,t2;

		t1= *(ap++);
		t2= *rp;
		if (c)
			{
			c=(t2 >= ((~t1)&BN_MASK2));
			t2=(t1+t2+1)&BN_MASK2;
			}
		else
			{
			t2=(t1+t2)&BN_MASK2;
			c=(t2<t1);
			}
		*(rp++)=t2;
		}
	/* there will be no carry */

	r->top=max;
	if (r->d[max-1] == 0) r->top--;
	return(1);
	}
